/**************************************************************************
**  (c) Copyright 1999, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : date.h
**      SYSTEM NAME    : ACL - Andromeda Class Library
**      VERSION NUMBER : 0.3.4
**
**  DESCRIPTION      :  
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Feb 06, 1998
**      LAST UPDATE     : Aug 08, 2020
**      MODIFICATIONS   : 
**************************************************************************/


#ifndef ACL_DATE_H
#define ACL_DATE_H

#include "String.h"

class date
{
   unsigned char  month, day;
   short year;

   long julian();
   date add(date D);
   date add(unsigned long days);

   date subtract(unsigned long days);

   friend class UTC;

public:

   date()
   {
      month = 0;
      day   = 0;
      year  = 0;
   }       
   
   date(unsigned  d, unsigned  m=0, short y=0)
   {
      day   = d;
      month = m;
      year  = y;
   }

   date(String s);

   bool proper();   // Check wether this is a proper calendar date

   friend date operator+(date, date);
   friend date operator+(unsigned long,  date);
   friend date operator+(date,  unsigned long);

   date operator += (date D)
   {
      return add(D);
   }

   date operator += (unsigned long l)
   {
      return add(l);
   }

   date operator ++ ()        // prefix
   {
      return add(1);
   }

   date operator ++ (int)     // postfix
   {
      return add(1);
   }

   friend date operator-(unsigned long,  date);
   friend date operator-(date,  unsigned long);
   friend long operator-(date&, date&);

   date operator -= (unsigned long l)
   {
      return subtract(l);
   }

   date operator -- ()        // prefix
   {
      return subtract(1);
   }

   date operator -- (int)     // postfix
   {
      return subtract(1);
   }

//   long(date);        // Calculate nr. of days since...

   int operator==(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm == 0;
   }

   int operator!=(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm != 0;
   }             
   
   int operator<=(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm <= 0;
   }             
   
   int operator>=(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm >= 0;
   }             
   
   int operator>(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm > 0;
   }             
   
   int operator<(date d)
   {
      int cm;
      
      cm = this->year - d.year;
      cm = (cm != 0) ? cm : this->month - d.month;
      cm = (cm != 0) ? cm : this->day   - d.day;
      
      return cm < 0;
   }

   //  Attributes.

   unsigned Day()
   {
      return day;
   }

   unsigned Month()
   {
      return month;
   }

   String MonthName(void);

   int Year()
   {
      return year;
   }
   //  Operations.

   int Leap(void);
   unsigned DaysInMonth();
   unsigned YearDay();
   int WeekDay()
   {
      return julian() % 7 + 1;
   }
    // TODO:YearDay WeekDay WeekNumber DayName

//   date operator*(double)
//   date& operator*=(double)
//   date operator/(double)
//   date& operator/=(double)

   friend std::ostream& operator<<(std::ostream&, const date&);
   friend std::istream& operator>>(std::istream&, date&);

   String format(const char *fmt = "%F");
};

date today(bool localtime = true); 

class hour
{
   int   hours;
   short minutes, seconds;

   friend class UTC;

   int compare(hour h)
   {
      int cm;
      
      cm = this->hours - h.hours;
      cm = (cm != 0) ? cm : this->minutes - h.minutes;
      cm = (cm != 0) ? cm : this->seconds - h.seconds;
      
      return cm;
   }

   hour increase();
   hour decrease();

public:

   hour()
   {
      hours = 0;
      minutes = 0;
      seconds = 0;
   }

   hour(int hr, short minute, short sec)
   {
      hours = hr;
      minutes = minute;
      seconds = sec;
   }

   hour(long sec)
   {
      hours = sec / 3600;
      minutes = sec / 60 % 60;
      seconds = sec % 60;
   }

   hour(String s);

   operator long()
   {
      return hours * 3600 + minutes * 60 + seconds;
   }

   bool proper();   // Check wether this is a proper clock time

   //  Attributes.

   short Second()
   {
      return seconds;
   }

   short Minute()
   {
      return minutes;
   }

   int Hour()
   {
      return hours;
   }

   bool operator==(hour h)
   {
      return compare(h) == 0;
   }

   bool operator!=(hour h)
   {
      return compare(h) != 0;
   }

   bool operator<(hour h)
   {
      return compare(h) < 0;
   }

   bool operator>(hour h)
   {
      return compare(h) > 0;
   }

   bool operator<=(hour h)
   {
      return compare(h) <= 0;
   }

   bool operator>=(hour h)
   {
      return compare(h) >= 0;
   }

   friend hour operator+(hour, hour);
   friend hour operator+(hour, long);
   hour operator += (hour h);

   hour operator ++ ()        // prefix
   {
      return increase();
   }

   hour operator ++ (int)     // postfix
   {
      return increase();
   }

   friend hour operator-(hour, hour);
   friend hour operator-(hour, long);
   hour operator -= (hour h);

   hour operator -- ()        // prefix
   {
      return decrease();
   }

   hour operator -- (int)     // postfix
   {
      return decrease();
   }


   friend std::ostream& operator<<(std::ostream &, const hour &);
   friend std::istream& operator>>(std::istream &, hour &);

   String format(const char *fmt = "%T");
};

hour now(bool localtime = true);

class UTC
{
   hour  t;
   date  d;

public:

   UTC()
   {
      /* Leave everything to the default hour and date constructors */
   }

   UTC(const String s);
   UTC(const time_t clock);
   UTC(date dt, hour tm)
   {
      d = dt;
      t = tm;
   }

   operator date()
   {
      return d;
   }

   operator hour()
   {
      return t;
   }

   time_t to_time_t();

   bool proper()
   {
      return d.proper() && t.proper();
   }

   bool operator==(UTC u)
   {
      return d == u.d && t == u.t;
   }

   bool operator!=(UTC u)
   {
      return d != u.d || t != u.t;
   }

   bool operator<=(UTC u)
   {
      if (d == u.d)
      {
         return t <= u.t;
      }
      else
      {
         return d <= u.d;
      }
   }

   bool operator<(UTC u)
   {
      if (d == u.d)
      {
         return t < u.t;
      }
      else
      {
         return d < u.d;
      }
   }

   bool operator>=(UTC u)
   {
      if (d == u.d)
      {
         return t >= u.t;
      }
      else
      {
         return d >= u.d;
      }
   }

   bool operator>(UTC u)
   {
      if (d == u.d)
      {
         return t > u.t;
      }
      else
      {
         return d > u.d;
      }
   }

   friend UTC operator+(UTC , UTC );
   UTC operator += (UTC h);

   friend long operator-(UTC , UTC );

   friend std::ostream& operator<<(std::ostream &, const UTC &);
   friend std::istream& operator>>(std::istream &, UTC &);

   String format(const char *fmt = "%F %T");
};

UTC Now(bool localtime = true);

#endif /* ACL_DATE_H */
