/**************************************************************************
**  (c) Copyright 1999, Andromeda Technology & Automation
***************************************************************************
** MODULE INFORMATION *
***********************
**      FILE NAME      : utc.cpp
**      SYSTEM NAME    : ACL - Andromeda Class Library
**      VERSION NUMBER : 0.3.4
**
**  DESCRIPTION      :  
**
**  EXPORTED OBJECTS : 
**  LOCAL    OBJECTS : 
**  MODULES  USED    :
***************************************************************************
**  ADMINISTRATIVE INFORMATION *
********************************
**      ORIGINAL AUTHOR : Arjen Baart - arjen@andromeda.nl
**      CREATION DATE   : Sep 16, 2002
**      LAST UPDATE     : Aug 08, 2020
**************************************************************************/


#include <time.h>

#include "date.h"
#include "parsedate.h"

UTC::UTC(String s)
{
   struct parseddate *pd;

   if (~s == 0)
   {
      t.hours   = 0;
      t.minutes = 0;
      t.seconds = 0;
      d.year   = 0;
      d.month  = 0;
      d.day    = 0;
   }
   else
   {
      pd = parsedate(s);
      t.hours = pd->hour;
      t.minutes = pd->minute;
      t.seconds = pd->second;
      d.year = pd->year;
      d.month = pd->month;
      d.day   = pd->day;
   }

   if (t.hours == -1)
      t.hours = 0;
   if (t.minutes == -1)
      t.minutes = 0;
   if (t.seconds == -1)
      t.seconds = 0;
}

UTC::UTC(const time_t clock)
{
   struct tm   *tp;

   tp = localtime(&clock);
   t = hour(tp->tm_hour, tp->tm_min, tp->tm_sec);
   d = date(tp->tm_mday, tp->tm_mon+1, tp->tm_year+1900);
}

time_t UTC::to_time_t()
{
   struct tm   tp;
   time_t  clock;

   tp.tm_year = d.Year() - 1900;
   tp.tm_mon  = d.Month() - 1;
   tp.tm_mday = d.Day();
   tp.tm_hour = t.Hour();
   tp.tm_min  = t.Minute();
   tp.tm_sec  = t.Second();
   tp.tm_isdst = -1;

   clock = mktime(&tp);

   return clock;
}

UTC Now(bool local)
{
   long      clock;
   struct tm   *tp;
   date        d;
   hour        t;

   time(&clock);
   if (local)
   {
      tp = localtime(&clock);
   }
   else
   {
      tp = gmtime(&clock);
   }
   
   t = hour(tp->tm_hour, tp->tm_min, tp->tm_sec);
   d = date(tp->tm_mday, tp->tm_mon+1, tp->tm_year+1900);

   return UTC(d, t);
}

UTC operator+(UTC t1, UTC t2)
{
   UTC t = t1;

   t.d += t2.d;
   t.t += t2.t;

   return t;
}

UTC UTC::operator+=(UTC u)
{
   d += u.d;
   t += u.t;

   return *this;
}

long operator-(UTC u1, UTC u2)
{
   long   days;
   long   seconds;

   days = u1.d - u2.d;
   seconds = long(u1.t) - long(u2.t);

   return days * 24 * 60 * 60 + seconds;
}

std::ostream& operator<<(std::ostream &s, const UTC &t)
{
   s << t.d << " " << t.t;

   return s;
}

std::istream &operator>>(std::istream &s, UTC &t)
{
   s >> t.d >> t.t;

   return s;
}

String UTC::format(const char *fmt)
{
   String  s;
   char    buf[80];
   struct  tm  ut;

   ut.tm_sec = t.seconds;
   ut.tm_min  = t.minutes;
   ut.tm_hour = t.hours;
   ut.tm_year = d.year - 1900;
   ut.tm_mon  = d.month - 1;
   ut.tm_mday = d.day;

   strftime(buf, 80, fmt, &ut);
   s = buf;

   return s;
}

