
#include "xml.h"

/*
 *   class xml
 */

void xml::ParseFile(const char * filename)
{
   ctxt     = xmlNewParserCtxt();
   document = xmlCtxtReadFile(ctxt, filename, NULL, XML_PARSE_NOWARNING|XML_PARSE_NOERROR);
}

String xml::RootElementName(void)
{
   String       root_name;
   xmlNodePtr   root;

   if (document != NULL)
   {
      root = xmlDocGetRootElement(document);
      root_name = String((const char *)root->name);
   }

   return root_name;
}

/*
 *  class xml_node
 */

xml_node xml::RootNode(void)
{
   xmlNodePtr   root;

   root = xmlDocGetRootElement(document);

   return xml_node(root);
}

//  Find the nth child element by name (non-recursive)
xml_node xml_node::operator[](int i)
{
   xmlNodePtr np = n->children;

   while (np != NULL && i != 0)
   {
      i--;
      np = np->next;
   }

   return xml_node(np);
}

xml_node xml_node::FindChildElement(String name, int nth)
{
   xml_node element;
   xmlNodePtr node = n->children;

   while (element.n == NULL && node != NULL)
   {
      if (node->type == XML_ELEMENT_NODE && name == (const char *)node->name)
      {
         if (nth == 0)
         {
            element.n = node;
         }
         nth--;
      }
      node = node->next;
   }

   return element;
}


/*
 *   class xml_element
 */

std::vector<xml_element> xml_element::operator[](String tagname)
{
   std::vector<xml_element> elements_found;

   xmlNodePtr np = n->children;
   while (np != NULL)
   {
      if (np->type == XML_ELEMENT_NODE && tagname == (const char *)np->name)
      {
         xml_element e(np);
         elements_found.push_back(e);
      }
      np = np->next;
   }

   return elements_found;
}

String xml_element::attribute(String name)
{
   String value;

   struct _xmlAttr          *prop_list;
   prop_list = n->properties;

   while (prop_list != NULL)
   {
      if (name == (const char *)prop_list->name)
      {
         value = (const char *)prop_list->children->content;
      }
      prop_list = prop_list->next;
   }

   return value;
}

std::map<String, String> xml_element::attributes(void)
{
   std::map<String, String> attrs;
   struct _xmlAttr          *prop_list;

   prop_list = n->properties;

   while (prop_list != NULL)
   {
      String name, value;

      name = String((const char *)prop_list->name);
      value = String((const char *)prop_list->children->content);

      attrs[name] = value;

      prop_list = prop_list->next;
   }

   return attrs;
}
