#include <libxml/parser.h>   // usually in /usr/include/libxml2, see xml2-config
#include <vector>
#include <map>

#include "String.h"

class xml_node;

class xml
{
   xmlParserCtxtPtr ctxt;
   xmlDocPtr        document;

public:

   xml()
   {
      ctxt     = NULL;
      document = NULL;
   }

   ~xml()
   {
      if (document != NULL)
      {
         xmlFreeDoc(document);
         document = NULL;
      }
   }

   void ParseFile(const char * filename);

   void SaveFile(const char * filename)
   {
      xmlSaveFile(filename, document);
   }

   String RootElementName(void);
   xml_node RootNode(void);
  
};

class xml_node
{
protected:
   xmlNodePtr     n;

public:

   xml_node()
   {
      n = NULL;
   }

   xml_node(xmlNodePtr np)
   {
      n = np;
   }

   xml_node(xml &doc)
   {
      xml_node node;
      node = doc.RootNode();

      n = node.n;
   }

   bool is_a_node(void)
   {
      return n != NULL;
   }

   xml_node operator[](int n);

   String name(void)
   {
      String node_name;

      if (n != NULL)
      {
         node_name = String((const char *)n->name);
      }

      return node_name;
   }

   xmlElementType type(void)
   {
      xmlElementType nodetype;

      nodetype = xmlElementType(0);
      if (n != NULL)
      {
         nodetype = n->type;
      }

      return nodetype;
   }

   // Assumes one childnode of type TEXT
   // TODO collect content from multiple TEXT nodes (recursively ?)
   String content(void)
   {
      String node_content;

      if (n != NULL)
      {
         node_content = String((const char *)n->children->content);
      }

      return node_content;
   }

   //  Find the nth child element by name (non-recursive)
   xml_node FindChildElement(String name, int nth = 0);
};

class xml_element : public xml_node
{
public:
   xml_element() : xml_node()
   {
   }

   xml_element(xmlNodePtr np) : xml_node(np)
   {
   }

   xml_element(xml &doc) : xml_node(doc)
   {
   }

   std::vector<xml_element> operator[](String tagname);

   String attribute(String name);
   std::map<String, String> attributes(void);

};

