/*******************************************************
 *  Unit test for the SuperString class
 *
 * test split and join of String and SuperString
 ******************************************************
 *
 */

#include "String.h"
#include <assert.h>

void print_ss(SuperString ss)
{
   std::cout << "[";
   for (int i = 0; i < ~ss; i++)
   {
      if (i != 0)
         std::cout << ",";
      std::cout << "\"" << ss[i] << "\"";
   }
   std::cout << "]";
}

int main()
{
   String      s0("The_String_To_Split");
   SuperString ss0;

   ss0 = s0.split("_");

   std::cout << "The string " << s0 << " split on \"_\":\n";
   print_ss(ss0);
   std::cout << "\n";

   assert(ss0[0] == "The");
   assert(ss0[1] == "String");
   assert(ss0[2] == "To");
   assert(ss0[3] == "Split");

   String s1;

   s1 = ss0.join(" ");
   std::cout << "Joined with ' ': " << s1 << "\n";
   assert(s1 == "The String To Split");

   String s2("A&amp;B&amp;C");
   SuperString ss2 = s2.split("&amp;");   // "A" "B" "C"
   std::cout << "The string \"" << s2 << "\" split on \"&amp;\":\n";
   print_ss(ss2);
   std::cout << "\n";
   assert(~ss2 == 3);

   //  Split an empty string
   String s3("");
   SuperString ss3 = s3.split(",");
   std::cout << "The string \"" << s3 << "\" split on \",\":\n";
   print_ss(ss3);
   std::cout << "\n";
   assert(~ss3 == 1);

   //  Test consequtive separators
   String s4("3.14   42");   //  Note 3 spaces
   SuperString ss4 = s4.split(" ");
   std::cout << "The string \"" << s4 << "\" split on \" \":\n";
   print_ss(ss4);
   std::cout << "\n";

   assert(~ss4 == 4);
   assert(ss4[0] == "3.14");
   assert(ss4[1] == "");
   assert(ss4[2] == "");
   assert(ss4[3] == "42");

   //  Test a separator at the start and end of the string
   String s5("-aaa-bbb-");   //  Note 3 spaces
   SuperString ss5 = s5.split("-");
   std::cout << "The string \"" << s5 << "\" split on \"-\":\n";
   print_ss(ss5);
   std::cout << "\n";

   assert(~ss5 == 4);
   assert(ss5[0] == "");
   assert(ss5[1] == "aaa");
   assert(ss5[2] == "bbb");
   assert(ss5[3] == "");

   //  Test tokenize
   String s6("aaa...bbb");
   SuperString ss6 = s6.tokenize("."); 
   std::cout << "The string \"" << s6 << "\" tokenized on \".\":\n";
   print_ss(ss6);
   std::cout << "\n";
   assert(~ss6 == 2);
   assert(ss6[0] == "aaa");
   assert(ss6[1] == "bbb");

   return 0;
}

