/*******************************************************
 *  Unit test for the xml_element class
 *
 * test xml element access
 ******************************************************
 *
 */

#include "xml.h"
#include <assert.h>

int main()
{
   xml    doc;

   const char xml_file[] = "xml_test01.xml";

   std::cout << "Reading XML file " << xml_file << "\n";

   doc.ParseFile("xml_test01.xml");

   xml_element root_node(doc);

   std::cout << "Root element is " << root_node.name() << "\n";
   std::cout << "Root node type is " << root_node.type() << "\n";
   assert(root_node.name() == "doc");

   std::vector<xml_element> book, chapters;

   book = root_node["book"];
   assert(book.size() == 1);

   chapters = book[0]["chapter"];

   std::cout << "Found " << chapters.size() << " chapter elements.\n";

   std::vector<xml_element> report;

   report = root_node["report"];
   assert(report.size() == 0);

   // Find an element three levels deep
 
   std::vector<xml_element> children;
   xml_element              found_node;

   children = book[0]["titlepage"];
   found_node = children[0];

   assert(found_node.is_a_node() == true);
   children = found_node["title"];
   found_node = children[0];
   assert(found_node.is_a_node() == true);
   std::cout << "Name of title node = " << found_node.name() << "\n";
   std::cout << "Content of title node = " << found_node.content() << "\n";

   return 0;
}

